#include "IsoEngine.h"

#include <Logger.h>

#include "components.h"
#include <raylib.h>

IsoEngine::IsoEngine(int screenWidth, int screenHeight, Network* network)
    : screenWidth(screenWidth), screenHeight(screenHeight), network(network), sceneManager(registry) {}

IsoEngine::~IsoEngine() {
    Shutdown();
}

void IsoEngine::Initialize() {
    InitWindow(screenWidth, screenHeight, "IsoEngine Game");
    SetTargetFPS(60);

    if (network) {
        network->Initialize();
    }
    Logger::GetCoreLogger().AddOutputStream(&std::cout);
    Logger::GetAppLogger().AddOutputStream(&std::cout);
    CORE_LOG_INFO("Engine initialized!");
}

void IsoEngine::Run() {
    while (isRunning && !WindowShouldClose()) {
        Update();
        Render();

        if (network) {
            network->ReceiveData();
        }
    }
}

void IsoEngine::Shutdown() {
    sceneManager.RenderActiveScene();
    CloseWindow();

    if (network) {
        network->Shutdown();
    }

    CORE_LOG_INFO("Engine shut down");
}

void IsoEngine::Update() {
    if (IsKeyPressed(KEY_ESCAPE)) {
        isRunning = false;
    }

    sceneManager.UpdateActiveScene(GetFrameTime());
    sceneManager.UpdateUI(registry);
}

void IsoEngine::Render() {
    BeginDrawing();
    ClearBackground(RAYWHITE);

    sceneManager.RenderActiveScene();

    DrawText("IsoEngine Running!", 10, 10, 20, DARKGRAY);
    sceneManager.RenderUI(registry);
    DrawFPS(10, 100);
    EndDrawing();
}
