#include "network.h"
#include <enet.h>
#include "Logger.h"

class ENetClient : public Network {
public:
    ENetClient() : client(nullptr), peer(nullptr) {}

    void Initialize() override {
        if (enet_initialize() != 0) {
            CORE_LOG_FATAL("An error occurred while initializing ENet");
            throw std::runtime_error("An error occurred while initializing ENet");
        }

        client = enet_host_create(nullptr, 1, 2, 0, 0);
        if (!client) {
            CORE_LOG_FATAL("An error occurred while creating the ENet client");
            throw std::runtime_error("An error occurred while initializing ENet Client");
        }

        ENetAddress address;
        enet_address_set_host(&address, "localhost");
        address.port = 1234;

        peer = enet_host_connect(client, &address, 2, 0);
        if (!peer) {
            CORE_LOG_ERROR("No available peers for initiating connection");
        }

        CORE_LOG_TRACE("ENet client initialized and connected to server");
    }

    void Shutdown() override {
        if (peer) {
            enet_peer_disconnect(peer, 0);
            CORE_LOG_TRACE("Disconnected from peer");
        }
        enet_host_destroy(client);
        enet_deinitialize();
        CORE_LOG_TRACE("ENet client shut down");
    }

    void SendData(const void* data, size_t size) override {
        if (peer) {
            ENetPacket* packet = enet_packet_create(data, size, ENET_PACKET_FLAG_RELIABLE);
            enet_peer_send(peer, 0, packet);
            CORE_LOG_TRACE("Data sent to server");
        }
    }

    void ReceiveData() override {
        ENetEvent event;
        while (enet_host_service(client, &event, 0) > 0) {
            if (event.type == ENET_EVENT_TYPE_RECEIVE) {
                CORE_LOG_TRACE("Received packet: {}", std::string(reinterpret_cast<const char*>(event.packet->data)));
                enet_packet_destroy(event.packet);
            }
        }
    }

private:
    ENetHost* client;
    ENetPeer* peer;
};
