//
// Created by illyum on 9/13/2024.
//

#ifndef FILEPACKER_H
#define FILEPACKER_H

#include <IsoEnginePCH.h>

#define BAP_MAGIC "BAP"
#define PACKER_VERSION 1

enum CompressionType {
    COMPRESSION_NONE,
    COMPRESSION_LZMA
};

enum FileType {
    PNG,
    RAWTEXT,
    RAWBINARY
};

struct FileEntry {
    std::string filePath;
    FileType fileType;
    CompressionType compression;
    size_t dataSize;
    size_t compressedSize;
    std::string metadata;
    size_t dataOffset;
};

struct BapHeader {
    char magic[3];
    int version;
    int fileCount;
    std::vector<FileEntry> fileEntries;
};

class FilePacker {
public:
    bool Pack(const std::string& directoryPath, const std::string& outputBapFile);
    bool Unpack(const std::string& bapFile, const std::string& outputDirectory);
    std::vector<FileEntry> listFiles(const std::string& bapFile);
    bool ExtractFile(const std::string& bapFile, const std::string& fileName, const std::string& outputPath);

private:
    void WriteHeader(std::ofstream& outFile, const BapHeader& header);
    BapHeader ReadHeader(std::ifstream& inFile);
    std::vector<char> CompressData(const std::vector<char>& input, CompressionType compression);
    std::vector<char> DecompressData(const std::vector<char>& input, CompressionType compression);
};

#endif //FILEPACKER_H
