//
// Created by illyum on 9/13/2024.
//

#include "FatalHandler.h"
#include <windows.h>
#include <dbghelp.h>
#include <fstream>
#include <sstream>
#include <format>

#pragma comment(lib, "dbghelp.lib")

namespace IsoEngine {
    template<typename... Args>
    void FatalHandler(const std::string &format, Args... args) {
        std::ostringstream stream;
        stream << std::format(format, args...);

        // TODO: change the name of the file to reflect the time the crash happened
        std::ofstream crashFile("crash.log", std::ios::app);
        crashFile << "FATAL ERROR: " << stream.str() << std::endl;

        HANDLE process = GetCurrentProcess();
        SymInitialize(process, NULL, TRUE);

        CONTEXT context;
        RtlCaptureContext(&context);

        STACKFRAME64 stackFrame = {};
        stackFrame.AddrPC.Offset = context.Rip;
        stackFrame.AddrPC.Mode = AddrModeFlat;
        stackFrame.AddrFrame.Offset = context.Rbp;
        stackFrame.AddrFrame.Mode = AddrModeFlat;
        stackFrame.AddrStack.Offset = context.Rsp;
        stackFrame.AddrStack.Mode = AddrModeFlat;

        crashFile << "Stack trace:\n";
        while (StackWalk64(IMAGE_FILE_MACHINE_AMD64, process, GetCurrentThread(), &stackFrame, &context, NULL,
                           SymFunctionTableAccess64, SymGetModuleBase64, NULL)) {
            DWORD64 address = stackFrame.AddrPC.Offset;
            if (address == 0) {
                break;
            }

            char buffer[sizeof(SYMBOL_INFO) + MAX_SYM_NAME * sizeof(TCHAR)];
            PSYMBOL_INFO symbol = (PSYMBOL_INFO) buffer;
            symbol->SizeOfStruct = sizeof(SYMBOL_INFO);
            symbol->MaxNameLen = MAX_SYM_NAME;

            DWORD64 displacement = 0;
            if (SymFromAddr(process, address, &displacement, symbol)) {
                crashFile << symbol->Name << " - 0x" << std::hex << symbol->Address << std::dec << "\n";
            } else {
                crashFile << "Unresolved symbol at address: 0x" << std::hex << address << std::dec << "\n";
            }
        }

        SymCleanup(process);
        crashFile.close();

        std::abort();
    }
}
